function [swat,pto,thetaB,Dprof,ZI]=Terracem_extract_swaths(p,a,station,profnum)
%=======================================================================
% extract swath profiles from rectangular boxed drwan in arcgis
% Designed by by Melnick and Jara-Mu?oz
% inputs: cropped xyz topography and polygon
% UTM projection only
% Extracts multiple swath profiles from a shapefile with several polygons
% inputs
% p: Vectors with the corners of clip poligons
% a: cropped xyz file
% station: site name
% profnum: profile number
%========================================================================

format long;
X=p(:,1); Y=p(:,2);
%Define coordinates for each vertice
x1=p(1,1); x2=p(2,1); x3=p(3,1); x4=p(4,1);
y1=p(1,2); y2=p(2,2); y3=p(3,2); y4=p(4,2);
%minimum and maximum for the entire dataset
minx=min(X); miny=min(Y);
maxx=max(X); maxy=max(Y);
nim=numel(p);
% LENGHT
% Alternative 1, calculate all diagonals, the middle value should be the lenght
A(1,1)=sqrt((x2-x1)^2+(y2-y1)^2);
A(2,1)=sqrt((x3-x2)^2+(y3-y2)^2);
A(3,1)=sqrt((x3-x4)^2+(y3-y4)^2);
A(4,1)=sqrt((x4-x2)^2+(y4-y2)^2);
A(5,1)=sqrt((x3-x1)^2+(y3-y1)^2);
A(6,1)=sqrt((x4-x1)^2+(y4-y1)^2);
% respective coordinate of each measured diagonal is added to A, (coordinate dragging)
A(1,2)=x1'; A(1,3)=y1'; A(1,4)=x2'; A(1,5)=y2';
A(2,2)=x2'; A(2,3)=y2'; A(2,4)=x3'; A(2,5)=y3';
A(3,2)=x3'; A(3,3)=y3'; A(3,4)=x4'; A(3,5)=y4';
A(4,2)=x2'; A(4,3)=y2'; A(4,4)=x4'; A(4,5)=y4';
A(5,2)=x1'; A(5,3)=y1'; A(5,4)=x3'; A(5,5)=y3';
A(6,2)=x1'; A(6,3)=y1'; A(6,4)=x4'; A(6,5)=y4';
% preparing data to obtain rectangle diagonals and angles
A(:,1)=round(A(:,1)); %eliminate decimal places in the first column, this cause problems with near 180? theta value
length=A(:,1);
X1=A(:,2); Y1=A(:,3);
X2=A(:,4); Y2=A(:,5);
H=[length X1 Y1 X2 Y2]; %defines the matrix H which will be used only for length calculation
L=H;     
% Calculation of the lengh of the longest side
% select the min side length
minV=min(H(:,1)); % find min value in H
H(H==minV)=0; % set all min values to zero
maxV=max(H(:,1)); % find max value in H
H(H==maxV)=0; % set all max values to zero
ind=find(H(:,1)~=0, 1, 'first'); %finds the address of the first non-zero value
ddg=H(ind); %lenght of the rectangle "Requested answer"
Dprof=ddg; %for use in sfitprof
% Find coordinate pairs of the shortest and nearest to the origin
G=sortrows(L,1);
minimum(1,:)=G(1,:);
minimum(2,:)=G(2,:);
%extract vertices coordinates of the shortest side nearest to the origin
x1cord=minimum(1,2);%pair x1,y1
y1cord=minimum(1,3);%pair x1,y1
x2cord=minimum(1,4);%pair x2,y2
y2cord=minimum(1,5);%pair x2,y2
x3cord=minimum(2,2);%pair x3,y3
y3cord=minimum(2,3);%pair x3,y3
x4cord=minimum(2,4);%pair x4,y4
y4cord=minimum(2,5);%pair x4,y4
%distance
De1=(x1cord^2)+(y1cord^2);
De2=(x2cord^2)+(y2cord^2);
De3=(x3cord^2)+(y3cord^2);
De4=(x4cord^2)+(y4cord^2);
%detect shorter side
minimum(1,6)=De1;
minimum(1,7)=De2;
minimum(2,6)=De3;
minimum(2,7)=De4;
%second euclidian distance
minimum(1,8)=((minimum(1,6)^2)+(minimum(1,7)^2));
minimum(2,8)=((minimum(2,6)^2)+(minimum(2,7)^2));
% definition of the lower coordinates for the shortest side
Q=sortrows(minimum,8);
x1c=Q(1,2);
y1c=Q(1,3);
x2c=Q(1,4);
y2c=Q(1,5);
%determining x1 and x2, x1 must be the nearest point to the origin
QF(1,1)=x1c;
QF(1,2)=y1c;
QF(2,1)=x2c;
QF(2,2)=y2c;
%determination of which of the points of the shorter side is near to the
%origin
DF=sortrows(QF,2); % modified, now use Y as discriminant
x1cord=DF(1,1);
y1cord=DF(1,2);
x2cord=DF(2,1);
y2cord=DF(2,2);
% Rotation angle theta and the middle point of the shortest side 
if x1cord-x2cord ~= 0; %any case & 90?
    
    if ((y2cord>y1cord) && (x2cord>x1cord)); 
    g1=(atand((y2cord-y1cord)/(x2cord-x1cord))); %calculate angle of the shortest side positive slope
    theta=-g1+180; %requested angle theta for swath
    %point coordinate
    d1=(sqrt((y2cord-y1cord)^2+(x2cord-x1cord)^2))/2; %1/2 hypotenusa     
    xpto=x1cord+((cosd(g1))*d1); %check bug in g1
    ypto=y1cord+((sind(g1))*d1);%check bug in g1
    thetaB=(90+g1);
    pto=[xpto, ypto]; %requested angle (point location)

    else 
    g2=(atand((y1cord-y2cord)/(x2cord-x1cord))); %calculate angle of the shortest side with negative slope
    theta=g2; %theta requested answer for matrix rotation
    d2=(sqrt((y2cord-y1cord)^2+(x1cord-x2cord)^2))/2; %1/2 hypotenusa    
    xpto=x1cord+((cosd(g2+180))*d2); 
    ypto=y1cord-((sind(g2+180))*d2); 
    thetaB=(90-g2);    
    pto=[xpto, ypto]; %requested answer (point location)
        
    end    
end

if x1cord-x2cord == 0; %horizontal rectangle (180?)

    g1=(atand((y2cord-y1cord)/(x2cord-x1cord))); %calculate angle of the shortest side positive slope
    theta=-g1+180; %requested angle theta for swath
     %point coordinate
     d1=(sqrt((y2cord-y1cord)^2+(x2cord-x1cord)^2))/2; %1/2 hypotenusa     
     xpto=x1cord+((cosd(g1))*d1); 
     ypto=y1cord+((sind(g1))*d1);
     thetaB=(90+g1);
     pto=[xpto, ypto]; %requested angle (point location)          
end 
% rotation matrix
for i=1:(numel(a(:,1))-1); 
    aa = [a(i,1) ; a(i,2)]; %"aa" will be equal to i rows in the column 1, and the same for the column 2
    b = [cosd(theta)  -sind(theta) ; sind(theta)  cosd(theta)] * aa; %define b including the line angle
    ar=b';
    ro(i,1) = ar(1,1); 
    ro(i,2) = ar(1,2); 
    ro(i,3) = a(i,3); %keeps Z in the colum 3 intact
end
clear i

if exist('ro','var')==1;
%gridding rotated xyz to matrix
ddx=abs(ro(1,1)-ro(2,1)); %define ddx as the absolute value of the diference of the proyected x
ddy=abs(ro(1,2)-ro(2,2)); %define ddy as the absolute value of the diference of the proyected y
xlim_min=min(ro(:,1)); %define the boundaries of proyected x coordinate
xlim_max=max(ro(:,1));
ylim_min=min(ro(:,2)); %define the boundaries of proyected Y coordinate
ylim_max=max(ro(:,2));
x=(xlim_min:ddx:xlim_max); %define x and y vectors
y=(ylim_min:ddy:ylim_max);
[XI,YI]=meshgrid(x,y); %mesh with x and y
ZI=griddata(ro(:,1),ro(:,2),ro(:,3),XI,YI,'nearest'); %grid rotated data with nearest neighbors 
% Swath profile extraction
ma=max(ZI'); %maximum raster value
me=mean(ZI'); %mean raster value
mi=min(ZI'); %minimum raster value
% use projected latitude as x
[ny nx]=size(ZI);
% use km as x
xx2=0:ddg/(ny-1):ddg;%X for normal swath

% DEFINE VARIABLE swat 
%and check errors during run
try
swat(:,1)=xx2(1,:)';
swat(:,2)=mi(1,:)';
swat(:,3)=me';
swat(:,4)=ma';
swat(:,5)=abs(atand((gradient(ma))));%,mean(diff(xx2')))))); %slope
catch
    msgbox('ERROR, this error occurs by one of the following reasons: 1)Some profiles are outside the topography extent. 2) The rectangles or are not 90 degrees at their corners (e.g. rectangles not made with the rectangle tool in ArcGIS(R)). 3) The rectangles are too thin and not encompases enough raster cells to calculate statistics. See Jara et al., (2015) in Geosphere to understand how to select the better swath profile width.')
end

h2=figure;
% plot swaths
subplot(3,1,1); 
hold on
U7=area(swat(:,1),swat(:,4));
U6=area(swat(:,1),swat(:,2));
set(U7,'FaceColor',[0.6 0.6 0.6],'edgeColor',[0.6 0.6 0.6]);
set(U6,'FaceColor',[1 1 1],'edgeColor',[0.6 0.6 0.6]);
set(gca,'Layer','top');
plot(swat(:,1),swat(:,3),'-k','linewidth',1.4);
axis([0 ddg min(swat(:,2)) max(swat(:,4))]);
xlabel('Distance along swath [m]');
ylabel('Elevation [m]');
tit=sprintf('Station %s, Profile %u',station,profnum);
title(tit);
grid on, box on
xlim([0 ddg]);

subplot(3,1,2); 
plot(swat(:,1),swat(:,5),'.r'); 
grid on, box on
xlim([0 ddg]);
ylabel('Slope (deg)'); xlabel('Distance along swath (m)');

subplot(3,1,3)
pcolor(YI,XI,ZI);
colormap (demcmap(ZI));
colorbar('location','east');
shading flat;

close(h2)
else   
msgbox('The topography not match with the extent of swath profiles, check input topography (..._dem.asc) or profiles (..._clip.shp), if the problem persist try deleting the file ...._swaths.mat from the station directory')    
return
end