%% Terracem_Free-face function
function shoreline=Terracem_fxcliff(swat,station,profnum,shorenum,pto,thetaB,Dprof,slo,ra,varargin) 
%==================================================================
% D. Melnick 2012, modified by Jara-Munoz, 2015
%
% Intersect cliff's free-face and interpolated abrasion platform to 
% find shoreline angle.
% 
% v. 1.8
%==================================================================
%Check inputs

if length(varargin) > 0
  
   clif1 = varargin{1};
   plat1 = varargin{2};
   inputset=1;
else
   inputset=0;
end   
% Set free-face slope and error
fface(1,1)=ra; %Plio-Pleist. Konya limestone (70 5) (slo ra)
fface(1,2)=slo;
format long
%load swath profile values
x=swat(:,1); %distance along profile
mi=swat(:,2); %min values%
me=swat(:,3); %mean values
ma=swat(:,4); %max values
slp=swat(:,5); %slope values

% Check if flipped, high part must be at right
if ma(1)>ma(end)
   type_profile=2; %identifies if profile is flipped
    
   x=max(x)-x+1;   
else   type_profile=1; %identifies if profile is in correct position
 
    x=x;
end

%plot of topographic distributions from swaths
%check batch mode

    
f1=figure(1);   
%plot topographic swaths
hold on
ax1=subplot(3,1,1:2);
hold on
plot(x,ma,'k');
plot(x,me,'color',[0.3 0.3 0.3]);
plot(x,mi,'color',[0.6 0.6 0.6]);
hleg1 = legend('Max topography','Mean topography','Min topography','Location','SouthEast');
xlim([min(x) max(x)]);
grid on 
box on
yl1=ylabel('Elevation (m)'); 
xl1=xlabel('Distance along profile (m)');
tit=sprintf('Station %s, Profile %u, Shoreline %u',station,profnum,shorenum);
title(tit);
%plot of slope of maximum topography
ax2=subplot (3,1,3);
hold on
plot(x,slp,'.k') ;
plot(x,moving(slp,50,'mean'),'r-');
grid on
box on
leyendita = legend('slope values','Location','NorthEast');
xlim([min(x) max(x)]); 
ylabel('Slope (deg)'); xlabel('Distance along profile (m)'); 
linkaxes([ax1 ax2], 'x'); 
% UNICONTROL RADIO BUTTON for select maximum or minimum topo for analysis
% Create the button group.
ht = uibuttongroup('visible','on','Position',[0 0 1 0.1],'Units','Normalized'); %radio buttons group
% Create two radio buttons inside the button group.
u1 = uicontrol('Style','radiobutton','String','Maximum swath topo','pos',[300 5 130 30],'parent',ht,'HandleVisibility','on');
u2 = uicontrol('Style','radiobutton','String','Minimum swath topo','pos',[100 5 130 30],'parent',ht,'HandleVisibility','on');
disp('Zoom in, select area, and press [Enter]')
%pause
set(ht, 'Visible','off') %hide buttons when drawing paleocliff and paleo platform
u1p=get(u1,'Value'); %detect max topo radiobutton handles
u2p=get(u2,'Value'); %%detect min topo radiobutton handles

%check which button was pushed and associate with type of topography used
if u1p==1;
answer=1; %max topography
else
answer=0; %minumum topography   
end

if inputset==1;

face=clif1;    
plat=plat1;  

else

if exist(sprintf ('%s_Clicks.mat',station),'file')==2;
load(sprintf ('%s_Clicks.mat',station));  %assumes that Clicks empty matrix was already saved  
else
Clicks=[]; %if clicks matrix not exist creates an empty  matrix for the next condition
end


%check structure existence
doesVarExist = true;
try
    Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).fix;
catch
    doesVarExist = false;
end

if doesVarExist == 1;
prompt1 = {'would you like to overwrite the data? Yes(0) / No (1)'};
dlg_title1 = 'rewrite or re-read the data?';
num_lines1 = 1;
def1 = {'1'}; %default values
answer1 = str2double(inputdlg(prompt1,dlg_title1,num_lines1,def1));

if answer1(1,1)==0; 
        existence=0; %parameter for data existence situation, existence=0 no previous data recognized.
        disp('enter outcrop position of paleocliff free face');
%input free-face
        [face(1,1),face(1,2)]=ginputc(1,'Color','r');       
        plot(face(1,1),face(1,2),'sk','markerfacecolor','k');
        disp('enter top and bottom of paleo abrasion platform');
        [plat(1,1),plat(1,2)]=ginputc(1,'Color','b');       
        [plat(2,1),plat(2,2)]=ginputc(1,'Color','b');       
else
        existence=1;  %parameter for data existence situation, existence=1 previous data recognized.  
        face=Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).fix.clif;
        plat=Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).fix.plat;
end
else
% input clif
existence=0; 
disp('enter outcrop position of paleocliff free face');
%input free-face
[face(1,1),face(1,2)]=ginputc(1,'Color','r');
disp('enter top and bottom of paleo abrasion platform');
[plat(1,1),plat(1,2)]=ginputc(1,'Color','b'); 
[plat(2,1),plat(2,2)]=ginputc(1,'Color','b');
end
end

if answer == 1
da(:,1)=x; 
da(:,2)=ma; 
md=swat; 
ny=length(md(:,1)); 

else
da(:,1)=x; 
da(:,2)=mi;
da(isnan(da)) = 0; %convert nan values to zero
md=swat; 
ny=length(md(:,1));     
end

%correct inverted profile
if type_profile==2
da=flipud(da);  
end

% select points for fitting cliff
daclif=da(da(:,1)>=face(1,1),:);
% select points for fitting platform
daplat1=da(da(:,1)<=plat(1,1),:);
daplat=daplat1(daplat1(:,1)>=plat(2,1),:);


%linear regressions
% free-face and cliff
xc=(daclif(:,1));
yc=daclif(1,2)-((daclif(:,1)-daclif(1,1))./tand(90+fface(1,1)));
yc1=daclif(1,2)-((daclif(:,1)-daclif(1,1))./tand(90+fface(1,1)+fface(1,2)));%+error
yc2=daclif(1,2)-((daclif(:,1)-daclif(1,1))./tand(90+fface(1,1)-fface(1,2))); %-error


%first order fit for cliff
[pclif,sclif]=polyfit(xc,yc,1);
[pclif1,sclif1]=polyfit(xc,yc1,1);
[pclif2,sclif2]=polyfit(xc,yc2,1);

% first order fit for platform
xp=daplat(:,1); yp=daplat(:,2);
[pplat,splat]=polyfit(xp,yp,1);

% extrapolate both
d=mean(diff(xc));
xx=min(xp):d:face(1,1); xx=xx';

[p_clif,d_clif]=polyval(pclif,xx,sclif);
[p_clif1,d_clif1]=polyval(pclif1,xx,sclif1);
[p_clif2,d_clif2]=polyval(pclif2,xx,sclif2);
[p_plat,d_plat]=polyval(pplat,xx,splat);
% locate intersection
mc=pclif(1,1); 
ic=pclif(1,2); 
mp=pplat(1,1); 
ip=pplat(1,2);
sh=[(-ic+ip)/(mc-mp),(ip*mc-ic*mp)/(mc-mp)];
shx=sh(1,1); 
shz=sh(1,2);

[shzp,shzep]=polyval(pplat,shx,splat);
% propagate 2s error by intersecting 2s regressions
% intersect max
mc=pclif1(1,1); ic=pclif1(1,2);
sh1=[(-ic+ip)/(mc-mp),(ip*mc-ic*mp)/(mc-mp)];
shx1=sh1(1,1); 
shz1=sh1(1,2);
[shzc1,shzec1]=polyval(pplat,shx1,splat);

% intersect min
mc2=pclif2(1,1); ic2=pclif2(1,2);
sh2=[(-ic2+ip)/(mc2-mp),(ip*mc2-ic2*mp)/(mc2-mp)];
shx2=sh2(1,1); 
shz2=sh2(1,2);
[shzc2,shzec2]=polyval(pplat,shx2,splat);

% calculate error
shze1=shz1-shzec1;
shze2=shz2+shzec2;
shze=(shze2-shze1);
close(f1);
% plot intersection
f2=figure;
clf
hold on
plot(x,ma,'-b'), grid on, box on %ma
plot(x,mi,'-k'), grid on, box on %ma
plot(xp,yp,'ok','markerfacecolor','b','MarkerSize',4)
plot(xx,p_plat,'r-',xx,p_plat+2*d_plat,'r--',xx,p_plat-2*d_plat,'r--') %platform
plot(xx,p_clif,'r-');
plot(xx,p_clif1,'r--');
plot(xx,p_clif2,'r--');

plot(face(1,1),face(1,2),'sk','markerfacecolor','r');
errorbar(shx,shz,shze,'ok','markerfacecolor','k');
axis([min(da(:,1)) max(da(:,1)) min(da(:,2)) max(da(:,2))]); 
ylabel('Elevation (m)'); xlabel('Distance along profnumile (m)');

v = axis;
xpos1=v(1)+((v(2)-v(1))/6);
ypos1=v(4)-((v(4)-v(3))/4);

clear v
text(xpos1,ypos1,sprintf('Cliff slope:\n%3.1f +/- %2.1f deg.\n SA: %6.2f +/- %4.2f m',fface(1,1),fface(1,2),shz,shze));
%text(min(da(:,1))+(max(da(:,1)-min(da(:,1))/5)),max(da(:,2))-max(da(:,2))/4,sprintf('Free-face slope\n%3.1f +/- %2.1f deg.\n\nShoreline angle\n elevation\n%6.2f +/- %4.2f m',fface(1,1),fface(1,2),shz,shze))
box on
yl1=ylabel('Elevation (m)'); 
xl1=xlabel('Distance along swath(m)');
%convert to utm
point2utm=Terracem_point2utm(sh,pto,thetaB, Dprof, type_profile); %call to point2utm function
Rx=point2utm(:,1); %UTM E
Ry=point2utm(:,2); %UTM N

if inputset~=1;
msgbox('SAVE FIT: Adjust zoom if necesary and push enter when ready to save')
pause 
end

if ishandle(f2) ==1
tit=sprintf('Station %s, Profile %u, Shoreline %u',station,profnum,shorenum);
titi=title(tit);
v = axis;
xpos=v(1)+(v(2)-v(1))/2;
ypos=v(4)-((v(4)-v(3))/14);
set(titi,'Position',[xpos ypos]);
legend('off');
%delete(tx1);
delete(yl1)%delete ylabel
delete(xl1)%elete xlabel
% The line below converts the current figure handle into a struct.
this_fig = handle2struct(gcf); 
else
msgbox('ATTENTION: Your shoreline was not saved, you must push enter to save your shoreline data')
end

%load &/or combine structures
if exist((sprintf ('%s_fits.mat',station)),'file')==2  
load(sprintf ('%s_fits.mat',station))
Fits.(sprintf('%s_%u_%u_fit',station,profnum,shorenum))=this_fig;
save((sprintf ('%s_fits.mat',station)),'Fits') 
else   
Fits.(sprintf('%s_%u_%u_fit',station,profnum,shorenum))=this_fig;
save((sprintf ('%s_fits.mat',station)),'Fits') 
end
close(findobj('type','figure','name','this_fig'))
close(findobj('type','figure','name','f2'))
close(findobj('type','figure','name','figure(2)'))
close(figure)
close(f2)
%Primary output
shoreline=[shorenum Rx Ry shx shz shze]; %number of shoreline,E, N, distance along profile,z ze
%Clicks output
%check if table exist
if exist((sprintf ('%s_Clicks.mat',station)),'file')==2 
%load table & combine structures
load(sprintf ('%s_Clicks.mat',station))
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).fix.clif=face;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).fix.plat=plat;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).fix.inputs.slo=slo; 
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).fix.inputs.ra=ra; 

%save the final combination
save((sprintf ('%s_Clicks.mat',station)),'Clicks') 
else
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).fix.clif=face;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).fix.plat=plat; 
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).fix.inputs.slo=slo; 
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).fix.inputs.ra=ra; 
save((sprintf ('%s_Clicks.mat',station)),'Clicks');       
end