%% Staircase analysis function
function shoreline=Terracem_sfitprof(swat,station,profnum,shorenum,pto,thetaB,Dprof,batchmode,varargin)
%=========================================================================
%Sfitprof (staircase analysis)
%Designed by Daniel Melnick and Julius Jara-Munoz
%inputs: swath results
%=========================================================================

if length(varargin) > 0
   batchmode=1; 
   clif1 = varargin{1};
   plat1 = varargin{2};
   inputset=1;
else
   inputset=0;
end   

resp=shorenum; 
format long
%load swath profile values
x=swat(:,1); %distance along profile
mi=swat(:,2); %min values%
me=swat(:,3); %mean values
ma=swat(:,4); %max values
slp=swat(:,5); %slope values

% Check if flipped, high part must be at right
if ma(1)>ma(end)
   type_profile=2; %identifies if profile is flipped
    
   x=max(x)-x+1;   
else   type_profile=1; %identifies if profile is in correct position
 
    x=x;
end

%check batch mode
if batchmode==1;
scrsz = get(0,'ScreenSize'); 
f1=figure(1);
set(f1,'Position',[scrsz(4)/2 scrsz(4)/2 scrsz(3)/2 scrsz(4)/2])
%clf, hold on
%f=f+1;
else
    
 f1=figure(1);   
end
%plot of topographic distributions from swaths
hold on
ax1=subplot(3,1,1:2);
hold on
plot(x,ma,'k')
plot(x,me,'color',[0.3 0.3 0.3])
plot(x,mi,'color',[0.6 0.6 0.6])
hleg1 = legend('Max topography','Mean topography','Min topography','Location','SouthEast');
xlim([min(x) max(x)]);
grid on 
box on
yl1=ylabel('Elevation (m)'); 
xl1=xlabel('Distance along profile (m)');
tit=sprintf('Station %s, Profile %u, Shoreline %u',station,profnum,shorenum);
title(tit);
%plot of slope of maximum topography
ax2=subplot (3,1,3);
hold on
plot(x,slp,'.k') 
plot(x,moving(slp,50,'mean'),'r-');
grid on
box on
leyendita = legend('slope values','Location','NorthEast');
xlim([min(x) max(x)]); 
ylabel('Slope (deg)'); xlabel('Distance along profile (m)'); 
linkaxes([ax1 ax2], 'x'); 
% UNICONTROL RADIO BUTTON for select maximum or minimum topo for analysis
% Create the button group.
ht = uibuttongroup('visible','on','Position',[0 0 1 0.1],'Units','Normalized'); %radio buttons group
% Create two radio buttons inside the button group.
u1 = uicontrol('Style','radiobutton','String','Maximum swath topo','pos',[300 5 130 30],'parent',ht,'HandleVisibility','on');
u2 = uicontrol('Style','radiobutton','String','Minimum swath topo','pos',[100 5 130 30],'parent',ht,'HandleVisibility','on');
% zoom into interest area
if batchmode==1;
pause(0.5)    
else
disp('Zoom in, select area, and press [Enter]')
%pause
end

set(ht, 'Visible','off') %hide buttons when drawing paleocliff and paleo platform
u1p=get(u1,'Value'); %detect max topo radiobutton handles
u2p=get(u2,'Value'); %%detect min topo radiobutton handles
%check which button was pushed and associate with type of topography used
if u1p==1
answer=1; %max topography
else
answer=0; %minumum topography   
end

% CHECK previous clicks routine 
% GUI backup system of results
if inputset==1;

clif=clif1;    
plat=plat1;    
    
else    
if batchmode==1;
existence=0; 
disp('enter top and bottom of paleocliff extrapolation line');
[clif(1,1),clif(1,2)]=ginputc(1,'Color','r'); 
[clif(2,1),clif(2,2)]=ginputc(1,'Color','r');
disp('enter top and bottom of paleo abrasion platform');
[plat(1,1),plat(1,2)]=ginputc(1,'Color','b'); 
[plat(2,1),plat(2,2)]=ginputc(1,'Color','b');    
else
        
if exist(sprintf ('%s_Clicks.mat',station),'file')==2
load(sprintf ('%s_Clicks.mat',station));  %assumes that Clicks empty matrix was already saved  
else
Clicks=[]; %if clicks matrix not exist creates an empty  matrix for the next condition
end

%check structure existence
doesVarExist = true;
try
    Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).class
catch
    doesVarExist = false;
end

if doesVarExist == 1
prompt1 = {'would you like to overwrite the data? Yes(0) / No (1)'};

dlg_title1 = 'rewrite or re-read the data?';
num_lines1 = 1;

def1 = {'1'}; %default values
answer1 = str2double(inputdlg(prompt1,dlg_title1,num_lines1,def1));

        if answer1(1,1)==0; 
        existence=0; %parameter for data existence situation, existence=0 no previous data recognized.
        disp('enter top and bottom of paleocliff extrapolation line');
        [clif(1,1),clif(1,2)]=ginputc(1,'Color','r'); 
        [clif(2,1),clif(2,2)]=ginputc(1,'Color','r');
        disp('enter top and bottom of paleo abrasion platform');
        [plat(1,1),plat(1,2)]=ginputc(1,'Color','b'); 
        [plat(2,1),plat(2,2)]=ginputc(1,'Color','b');
    
        else
        existence=1;  %parameter for data existence situation, existence=1 previous data recognized.  
        clif=Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).class.clif;
        plat=Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).class.plat;
        end

else
% input clif
existence=0; 
disp('enter top and bottom of paleocliff extrapolation line');
[clif(1,1),clif(1,2)]=ginputc(1,'Color','r'); 
[clif(2,1),clif(2,2)]=ginputc(1,'Color','r');
disp('enter top and bottom of paleo abrasion platform');
[plat(1,1),plat(1,2)]=ginputc(1,'Color','b'); 
[plat(2,1),plat(2,2)]=ginputc(1,'Color','b');
end
end
end
% select cliff and plat for fitting
%choose maximum or minimum swath topography for analysis
%minimum topography is used when dune depositation is observed above the
%surface / maximum topography is used when erosion processes dominates

if answer == 1
da(:,1)=x; 
da(:,2)=ma; 
md=swat; 
ny=length(md(:,1)); 
else
da(:,1)=x; 
da(:,2)=mi; 
md=swat; 
ny=length(md(:,1));     
end

% delimitate area for interpolation
j=1; k=1;
for i=1:ny
if (da(i,1)>clif(2,1)) && (da(i,1)<clif(1,1))
    daclif(j,:)=da(i,:);
    j=j+1;
end
if (da(i,1)>plat(2,1)) && (da(i,1)<plat(1,1))
    daplat(k,:)=da(i,:);
    k=k+1;
end
end

% fit cliff
xc=daclif(:,1); yc=daclif(:,2);
[pclif,sclif]=polyfit(xc,yc,1);
[p_clif,d_clif]=polyval(pclif,xc,sclif);
% fit plat
xp=daplat(:,1); yp=daplat(:,2);
[pplat,splat]=polyfit(xp,yp,1);
[p_plat,d_plat]=polyval(pplat,xp,splat);
% save points and projected profile
clifx=[clif(1,1),clif(1,2),clif(2,1),clif(2,2)];
platx=[plat(1,1),plat(1,2),plat(2,1),plat(2,2)];

d=xc(2)-xc(1);
d=abs(d);
xx=min(xp):d:max(xc); xx=xx';
[p_clif,d_clif]=polyval(pclif,xx,sclif);
[p_plat,d_plat]=polyval(pplat,xx,splat);
close(f1)
% plot intersection
f2=figure(2);
grid off
hold on
plot(x,ma,'-b'), grid on, box on %ma
plot(x,mi,'-k'), grid on, box on %ma
plot(xc,yc,'ok','MarkerSize',5,'MarkerFaceColor','b')
plot(xx,p_clif,'r-',...
   xx,p_clif+2*d_clif,'r--',xx,p_clif-2*d_clif,'r--') %, axis equal tight
hold on
%plot(da(:,1),da(:,2),'+k')
plot(xp,yp,'ok','MarkerSize',5,'MarkerFaceColor','b')
plot(xx,p_plat,'r-',...
   xx,p_plat+2*d_plat,'r--',xx,p_plat-2*d_plat,'r--') 
axis([min(xx)-15 max(xx)+15 min(yp)-15 max(yc)+15]); 
hleg=legend('Max topography','Min topography','Platform and cliff points','linear fit','2s range','location','northwest');
yl1=ylabel('Elevation (m)'); 
xl1=xlabel('Distance along swath(m)');
% find intersections
mc=pclif(1,1); mp=pplat(1,1); ic=pclif(1,2); ip=pplat(1,2);
% output shoreline angle is sh
sh=[(-ic+ip)/(mc-mp),(ip*mc-ic*mp)/(mc-mp)];
shx=sh(1,1); %distance along ptofile
shz=sh(1,2); %elevation
[shzp,shzep]=polyval(pplat,shx,splat);
[shzc,shzec]=polyval(pclif,shx,sclif);
shze=(shzep+shzec); %error in z
shx1=shx+shze;
str1=num2str(shz,3);
str2=num2str(shze,2);
%text on plot

%tx1=text(shx1-2*(max(xx)-min(xx))/3,shz+(max(yp)-min(yp)/2),sprintf('SA: %s +/- %s m',str1,str2),'FontSize',12,'BackgroundColor',[1 1 1]); %Shoreline angle value and location
tx1=text(shx1-(max(xx)-min(xx))/4,shz+(max(yp)-min(yp)/2),sprintf('SA: %s +/- %s m',str1,str2),'FontSize',12,'BackgroundColor',[1 1 1]); %Shoreline angle value and location

%text(shx,shz,sprintf('n=%u',shz,'n=+-%u',shze,'FontSize',16)) %fix me
%obtain the UTM coordinate pairs of shoreline
point2utm=Terracem_point2utm(sh,pto,thetaB, Dprof, type_profile); %call to point2utm function
Rx=point2utm(:,1); %UTM E
Ry=point2utm(:,2); %UTM N
h=errorbar(shx,shz,shze,'ok','MarkerFaceColor','k'); %plot shoreline angle with error in z
errorbar_tick(h, 30);
box on
ylabel('Elevation (m)'); xlabel('Distance along swath (m)');
%Figs output
%figure output as a nested structure
%check if zoom adjust is first necesary
if batchmode==1;
pause(0.5)      
tit=sprintf('Station %s, Profile %u, Shoreline %u',station,profnum,shorenum);
titi=title(tit);
v = axis;
xpos=v(1)+(v(2)-v(1))/2;
ypos=v(4)-((v(4)-v(3))/14);
set(titi,'Position',[xpos ypos]);
legend('off')
delete(tx1);
delete(yl1)%delete ylabel

xpos1=v(1)+((v(2)-v(1))/6);
ypos1=v(4)-((v(4)-v(3))/4);

clear v
%tx2=text(xpos1,ypos1,sprintf('SA: %u +/- %u m',shz,shze));
tx2=text(shx1-(max(xx)-min(xx))/4,shz+(max(yp)-min(yp)/2),sprintf('SA: %s +/- %s m',str1,str2),'FontSize',12,'BackgroundColor',[1 1 1]); %Shoreline angle value and location (2/3)
delete(xl1)%delete xlabel
   
else    
msgbox('SAVE FIT: Adjust zoom if necesary and PUSH ENTER when ready')

pause

% title
tit=sprintf('Station %s, Profile %u, Shoreline %u',station,profnum,shorenum);
titi=title(tit);
v = axis;
xpos=v(1)+(v(2)-v(1))/2;
ypos=v(4)-((v(4)-v(3))/14);
set(titi,'Position',[xpos ypos]);
legend('off')
delete(tx1);
delete(yl1)%delete ylabel
delete(xl1)%delete xlabel

tx1=text(shx1-2*(max(xx)-min(xx))/3,shz+(max(yp)-min(yp)/2),sprintf('SA: %s +/- %s m',str1,str2),'FontSize',12,'BackgroundColor',[1 1 1]); %Shoreline angle value and location
end
% The line below converts the current figure handle into a struct.
this_fig = handle2struct(gcf); 
%load &/or combine structures
if exist((sprintf ('%s_fits.mat',station)),'file')==2  
load(sprintf ('%s_fits.mat',station))
Fits.(sprintf('%s_%u_%u_fit',station,profnum,shorenum))=this_fig;
save((sprintf ('%s_fits.mat',station)),'Fits') 
else   
Fits.(sprintf('%s_%u_%u_fit',station,profnum,shorenum))=this_fig;
save((sprintf ('%s_fits.mat',station)),'Fits') 
end
close(findobj('type','figure','name','this_fig'))
close(findobj('type','figure','name','f2'))
close(findobj('type','figure','name','figure(2)'))
close(figure)
close(f2)
%Primary output
shoreline=[shorenum Rx Ry shx shz shze]; %number of shoreline,E, N, distance along profile,z ze
%Clicks output
%check if table exist
if exist((sprintf ('%s_Clicks.mat',station)),'file')==2    
%load table & combine structures
load(sprintf ('%s_Clicks.mat',station))
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).class.clif=clif;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).class.plat=plat;
%save the final combination
save((sprintf ('%s_Clicks.mat',station)),'Clicks') 
else
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).class.clif=clif;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).class.plat=plat;      
save((sprintf ('%s_Clicks.mat',station)),'Clicks')       
end
clear nny my y_bajo bajo yp y_alto alto yc