%% Stacks analysis function
function [stacks]=Terracem_stacks(swat,station,profnum,pto,thetaB,Dprof,peak,varargin)
%==================================================================
% Jara-Munoz (2015)
%Function designed to map shorline angles in rocky coasts, characterized by
%stacks and stumps
%==================================================================
%check inputs
if length(varargin) > 0
  
   quad1   = varargin{1};
   X1      = varargin{2};
   Y1      = varargin{3};
   XSYS1   = varargin{4};
   
   inputset=1;
else
   inputset=0;
end   

format long
Peack_search=(peak); %search ratio for peaks
x=swat(:,1); %distance along profile
mi=swat(:,2); %min values%
me=swat(:,3); %mean values
ma=swat(:,4); %max values
slp=swat(:,5); %slope values
% Check if flipped, high part must be at right
if ma(1)>ma(end)
   type_profile=2; %identifies if profile is flipped
    
   x=max(x)-x+1;   
else   type_profile=1; %identifies if profile is in correct position
 
    x=x;
end
%plot swath
f1=figure(1); 
clf, hold on
%plot of topographic curves
ax1=subplot(2,1,1);%new subplot
hold on
plot(x,ma,'k')
plot(x,me,'color',[0.3 0.3 0.3])
plot(x,mi,'color',[0.6 0.6 0.6])
hleg1 = legend('Max topography','Mean topography','Min topography','Location','SouthEast');
xlim([min(x) max(x)]);
box on
ylabel('Elevation (m)'); xlabel('Distance along profile (m)');
tit=sprintf('Station %s, Profile %u',station,profnum);
title(tit);
%plot of slope
ax2=subplot (2,1,2); 
hold on
plot(x,slp,'.k')
plot(x,moving(slp,50,'mean'),'r-');
leyendita = legend('slope values','Location','NorthEast');
xlim([min(x) max(x)]);
ylabel('Slope (deg)'); xlabel('Distance along profile (m)');
grid on 
box on
linkaxes([ax1 ax2], 'x'); %new
da(:,1)=x; %lenght
da(:,2)=ma; %depth
md=swat; 
ny=length(md(:,1)); 

if inputset==1;

quad=quad1;    

else

%CHECK previous clicks routine
if exist(sprintf ('%s_Clicks.mat',station),'file')==2
load(sprintf ('%s_Clicks.mat',station));  %assumes that Clicks empty matrix was already saved  
else
Clicks=[]; %if clicks matrix not exist creates an empty  matrix for the next condition
end
doesVarExist = true;
try
    Clicks.(sprintf('prof_%u',profnum)).stack  %check existance of the structure
catch
    doesVarExist = false;
end
if doesVarExist == 1  %if the structure exist
prompt1 = {'would you like to overwrite the data? Yes(0) / No (1)'};
dlg_title1 = 'rewrite or re-read the data?';
num_lines1 = 1;
def1 = {'1'}; %default values
answer1 = str2double(inputdlg(prompt1,dlg_title1,num_lines1,def1));

        if answer1(1,1)==0; 
        existence=0; %parameter for data existence situation, existence=0 no previous data recognized.
        disp('select area for peak detection (1st upper rigth corner then lower left)');
        [quad(1,1),quad(1,2)]=ginputc(1,'color','b'); 
        [quad(2,1),quad(2,2)]=ginputc(1,'color','b');            
        else
        existence=1;  %parameter for data existence situation, existence=1 previous data recognized.          
        quad=Clicks.(sprintf('prof_%u',profnum)).stack.quad;
        
        end    
else
existence=0; 
% input peak detection area
disp('select area for peak detection (1st upper rigth corner then lower left)');
[quad(1,1),quad(1,2)]=ginputc(1,'color','b'); 
[quad(2,1),quad(2,2)]=ginputc(1,'color','b');
end
end
%Isolate area for peak detection
j=1; k=1;
for i=1:ny
if (da(i,1)>quad(2,1)) && (da(i,1)<quad(1,1)) %reemplazando cliff
    daquad(j,:)=da(i,:);%
    j=j+1;    
end
end
clear i j
% detect peaks
xc=daquad(:,1); 
yc=daquad(:,2);
%Peak det function (Billauer,2009)
[maxtab, mintab] = peakdet(yc, Peack_search, xc); %peak search
%plot maximum peaks detected
hold on
plot(xc,yc,'k');
plot(maxtab(:,1), maxtab(:,2),'.g','MarkerEdgeColor',[1 0 1],'MarkerSize',8);

%Clasify peaks by terrace level 
G=maxtab(:,2);
nim=numel(G);
npts=sqrt(nim);
x1=min(G):((max(G)-min(G))/npts):max(G); %this is the X axis, Rank of values in the X axis (G), the bin rank ins defined in the center, between :: 
y=G;
n_elements = histc(y,x1); %Y axe of cumulative frecuency histogram
c_elements = cumsum(n_elements); %this is for cumulative frecuency plot (not used here)
clear  n_elements c_elements h npts G y x1 nim

if inputset==1;
X=X1;
Y=Y1;
fringes=numel(X)-1;

else
%Clasify peaks by terrace level 2
if existence==1
X=Clicks.(sprintf('prof_%u',profnum)).stack.XY(:,1);    
Y=Clicks.(sprintf('prof_%u',profnum)).stack.XY(:,2);  
fringes=numel(X)-1;
else
    
%Input DIALOG
pause(2) 
prompt = {'how many elevation fringes would you like to define? (maximum 3 fringes)'};
dlg_title = 'Input for peaks function';
num_lines = 1;
def = {'3'}; %default values
answer = str2double(inputdlg(prompt,dlg_title,num_lines,def));
fringes=answer(1,1);
disp('mark only the upper bound of each fringe')   
[X,Y]=ginputc(fringes,'color','g'); 
Y=vertcat(Y,zeros(1,1)); %add zeros file (for loop) 
X=vertcat(X,zeros(1,1));
end
end

maxi=max(x);
% clasify the matrix maxtab in the fringe ranges 
hold off
nh=length(maxtab(:,1)); 
for i=1:(fringes) 
    
    for h=1:nh    
    if (maxtab(h,2)<Y(i,1)) && (maxtab(h,2)>Y(i+1,1))    
        maxtab(h,3)=i;     
    end
    end
    
    line([0 maxi],[Y(i,:) Y(i,:)], 'Color', [0 1-(i/(fringes)) 0],'LineStyle','--')   
end
clear i h j maxi mintab md scrsz tit daquad def nh numlines prompt ax1 ax2 k nh ny yc xc f da 
% shoreline angle on terrace remains on the cliffs 
if inputset==1;
    
     if isempty(XSYS1)
            shore=[];
            XS=[];
            YS=[]; 
     else
         XS=XSYS1(:,1);
         YS=XSYS1(:,2);
         shore(:,1)=XS;
         shore(:,2)=YS;
         ns=numel(YS); 
         
     end

else
    
if existence==1
            if isempty(Clicks.(sprintf('prof_%u',profnum)).stack.XSYS)==1
            shore=[];
            XS=[];
            YS=[];
            else              
            XS=Clicks.(sprintf('prof_%u',profnum)).stack.XSYS(:,1); 
            YS=Clicks.(sprintf('prof_%u',profnum)).stack.XSYS(:,2);                
            shore(:,1)=XS;
            shore(:,2)=YS;
            ns=numel(YS); 
            end
else       
% mark the shoreline point in the screen according with the Fringe
disp ('mark shoreline angles, if not push ENTER')
[XS,YS]=ginputc(fringes,'color','k'); 
            if isempty(YS)==1
            shore=[];
            else
            shore(:,1)=XS;
            shore(:,2)=YS;
            ns=numel(YS); 
            end
end
end


if size(shore)==0
    maxtab=maxtab;
else   
% loop to automaticaly identify elevation fringe for each shoreline angle
%maximum number is 3 fringes
    for h=1:ns %1:number of shoreline angles selcted could be different that number of fringes in some cases    
        for i=1:fringes %number of fringes   
         if (shore(h,2)<Y(i,1)) && (shore(h,2)>Y(i+1,1))    
        shore(h,3)=i;                
        % Condition: if two shorelines angles have the same fringe range
        % move the minor one to next fringe
        %new condition to check for repeated fringe values in shorelines   
        numb=sum(shore(:,3)==i);             
         if numb==1 || numb==0            
          %shore(h,3)=i; 
          intplat(i).shore=shore(h,:);            
         else   % Move repeated to lower level fringe    
            sortshore=sort(shore,2,'descend');            
            %is not possible to use arbitrary asinged fringes
            %so, we detect repeated fringes to evaluate the next alternatives            
            tabl=tabulate(sortshore(:,3));
            c=max(tabl); %statistics of sortshore
            rept=c(:,1); %identified repeated value              
            %Alternative1 if repeated is 2 use this  (working good)
               if rept==2                   
            intplat(i).shore=sortshore(1,:);
            intplat(i).shore=sortshore(2,:);
            j=i+1;   
            intplat(j).shore=sortshore(3,:);
            intplat(j).shore(:,3)=j;       
               end                 
            %Alternative2 if repeated is 1 use this (working good)
               if rept==1         
            intplat(i).shore=sortshore(1,:);
            k2=i+1;    
            intplat(k2).shore=sortshore(2,:);
            intplat(k2).shore(:,3)=k2;                                      
               end          
         end   
         end  
        end 
    end   
clear i h 
hold on
plot(shore(:,1),shore(:,2),'.k','MarkerSize',10);
% create outputs table
% add shoreline angle data to peaks point matrix
maxtab=vertcat(intplat(1,:).shore,maxtab);
end
% clasify and dismember maxtab on each elevation fringe
for i = 1:fringes
plat(i).level = maxtab(maxtab(:,3) == i,:);
end
clear i sortshore rept c tabl numb shore ns 
% separated vectors and fit stacks in profile
% plot and function interplations
close(f1)
f2=figure(2);
hold on  
for i=1:fringes
   xpeak(:,1)=plat(1,i).level(:,1);
   ypeak(:,1)=plat(1,i).level(:,2);   
   % stack 
   [psurf,ssurf]=polyfit(xpeak,ypeak,1);
   dxm=1; %mean(diff(xpeak));%prepare to extend fit
   xxx=min(xpeak):dxm:max(x);
   xxx=xxx'; %range added to extend fit line   
   [p_surf,d_surf]=polyval(psurf,xxx,ssurf);
% save fiting and other data to table structure
   intplat(i).slope=psurf; 
   intplat(i).p_surf=p_surf;
   intplat(i).d_surf=d_surf;
   intplat(i).xpeak=xpeak(:,1); %x value for peaks
   intplat(i).ypeak=ypeak(:,1); %y value for peaks
   intplat(i).xxx=xxx; %x values for interpolation      
%extract fit-maxtopo intersection
m_rect=intplat(i).slope(1,1); %slope of interpolation rect (tand of rect angle)
%c value
c_rect=intplat(i).slope(1,2); %second coeficient of the rect ecuation 'c'
% check this part because replace x and ma por swat values
swati(:,1)=x;
swati(:,2)=ma;
for j=1:(numel(swati(:,1)));
if fix(m_rect*(swati(j,1))+(c_rect)) == nearest(swati(j,2)); %aproximation to topography    
    shore_int(:,1)=swati(j,1);
    shore_int(:,2)=swati(j,2);   
end
end 
   %take error from interpolation and store in the third row
   %take number of shorelines from the number of fringes in loop
   shore_int(:,3)=min(intplat(i).d_surf); %erro in Y
   shore_int(:,4)=i; %shorenum   
   %store level of each shoreline angle in the fourth row  
   %create structure table
   intplat(i).shore_int=shore_int; %projected shoreline point 
   %FIGURE FINAL    
   plot(x,ma,'-k'), 
   plot(x,me,'Color', [0.5 0.5 0.5]), 
   plot(x,mi,'Color', [0.7 0.7 0.7]), 
   xlim([min(x) max(x)]); 
   ylim([0 max(ma)])
   box on; 
   ylabel('Elevation (m)'); xlabel('Distance along profile (m)');
   plot(intplat(i).xxx,intplat(i).p_surf,'Color', [0+(i/(fringes)) 1-(i/(fringes)) 0],'LineStyle','--')
   plot(intplat(i).xpeak,intplat(i).ypeak,'.','Color', [0+(i/(fringes)) 1-(i/(fringes)) 0],'MarkerSize',20)
   errorbar(intplat(i).shore_int(:,1),intplat(i).shore_int(:,2),intplat(i).shore_int(:,3),'dk','markerfacecolor','k')   
   clear xpeak ypeak xxx dxm p_surf d_surf psurf ssurf shore_int     
end  
   %INSET
   G=maxtab(:,2);  
   nim=numel(G);
   npts=sqrt(nim);
   x1=min(G):((max(G)-min(G))/npts):max(G); %this is the X axis, Rank of values in the X axis (G), the bin rank ins defined in the center, between :: 
   y=G;
   n_elements = histc(y,x1); %Y axe of cumulative frecuency histogram
   c_elements = cumsum(n_elements);
    
ax3=axes('position',[0.15 0.7 0.15 0.2]);
   %%histogram of height
    hist((G),(npts));
    ylim([0 nim/1.5])    
    h = findobj(gca,'Type','patch');
    set(h,'EdgeColor',[0.9 0.9 0.9])
    set(gca, 'YTick', []);
    set(gca,'yaxislocation','right','tickdir','in')
    text((nim-(nim/8)),npts+(npts/1),sprintf('n=%u',nim))
    xlabel('Elevation (m)'); ylabel('Peaks frecuency')
    set(gcf,'PaperUnits','centimeters'); set(gcf, 'PaperType','A4');
    fout = sprintf('%s_%u_stack.pdf',station,profnum);
    saveas(gcf,fout,'pdf');
% outputs
shoreline_angles=vertcat(intplat(1,:).shore_int); %main output shorelineangles with error 
x_sh=shoreline_angles(:,1); %distance along profile
y_sh=shoreline_angles(:,2); %elevation
sh1(:,1)=x_sh; 
sh1(:,2)=y_sh;
e_sh=shoreline_angles(:,3);
num_sh=shoreline_angles(:,4);
all_peaks1=maxtab;
%obtaining the UTM coordinate pairs of shoreline angle
nimb=numel(shoreline_angles(:,4));
for q=1:nimb  
point2utm(q,:)=Terracem_point2utm(sh1(q,:),pto,thetaB, Dprof, type_profile); %call to point2utm function
Rx(q,1)=point2utm(q,1); %UTM E
Ry(q,1)=point2utm(q,2); %UTM N
end
%obtaining the UTM coordinate pairs for stack points (maxtab)
sh2(:,1)=all_peaks1(:,1);%peak distance along profile
sh2(:,2)=all_peaks1(:,2);%peak elevation
sh2(:,3)=all_peaks1(:,3);%peak code
nimc=numel(all_peaks1(:,1)); %number of peaks
for r=1:nimc   
point2utm(r,:)=Terracem_point2utm(sh2(r,:),pto,thetaB, Dprof, type_profile); %call to point2utm function
end
%Primary output table
stacks=[num_sh Rx Ry x_sh y_sh e_sh]; 
%Figures as structure output
if inputset~=1
msgbox('SAVE FIT: Adjust zoom and push ENTER when ready')
pause
end
% The line below converts the current figure handle into a struct.
this_fig = handle2struct(gcf); 
%load &/or combine structures
if exist((sprintf ('%s_fits.mat',station)),'file')==2  
load(sprintf ('%s_fits.mat',station))
Fits.(sprintf('%s_%u_1_fit',station,profnum))=this_fig;
save((sprintf ('%s_fits.mat',station)),'Fits') 
else   
Fits.(sprintf('%s_%u_1_fit',station,profnum))=this_fig;
save((sprintf ('%s_fits.mat',station)),'Fits') 
end

close(f2)
%Clicks data output
%check if table exist

if exist((sprintf ('%s_Clicks.mat',station)),'file')==2    
%load table & combine structures
load(sprintf ('%s_Clicks.mat',station))
Clicks.(sprintf('prof_%u',profnum)).stack.quad=quad;
Clicks.(sprintf('prof_%u',profnum)).stack.XY=[X,Y];
Clicks.(sprintf('prof_%u',profnum)).stack.XSYS=[XS,YS];
Clicks.(sprintf('prof_%u',profnum)).stack.inputs.peak=peak;
%save the final combination
save((sprintf ('%s_Clicks.mat',station)),'Clicks') 
else    
Clicks.(sprintf('prof_%u',profnum)).stack.quad=quad;
Clicks.(sprintf('prof_%u',profnum)).stack.XY=[X,Y];
Clicks.(sprintf('prof_%u',profnum)).stack.XSYS=[XS,YS]; 
Clicks.(sprintf('prof_%u',profnum)).stack.inputs.peak=peak;
%if table not exist, just save    
save((sprintf ('%s_Clicks.mat',station)),'Clicks') 
end
end



function [maxtab, mintab]=peakdet(v, delta, x)
%PEAKDET Detect peaks in a vector
%        [MAXTAB, MINTAB] = PEAKDET(V, DELTA) finds the local
%        maxima and minima ("peaks") in the vector V.
%        MAXTAB and MINTAB consists of two columns. Column 1
%        contains indices in V, and column 2 the found values.
%        With [MAXTAB, MINTAB] = PEAKDET(V, DELTA, X) the indices
%        in MAXTAB and MINTAB are replaced with the corresponding
%        X-values.
%        A point is considered a maximum peak if it has the maximal
%        value, and was preceded (to the left) by a value lower by
%        DELTA.

% Eli Billauer, 3.4.05 (Explicitly not copyrighted).
% This function is released to the public domain; Any use is allowed.
maxtab = [];
mintab = [];
v = v(:); % Just in case this wasn't a proper vector
if nargin < 3
  x = (1:length(v))';
else 
  x = x(:);
  if length(v)~= length(x)
    error('Input vectors v and x must have same length');
  end
end
  
if (length(delta(:)))>1
  error('Input argument DELTA must be a scalar');
end

if delta <= 0
  error('Input argument DELTA must be positive');
end
mn = Inf; mx = -Inf;
mnpos = NaN; mxpos = NaN;
lookformax = 1;

for i=1:length(v)
  this = v(i);
  if this > mx, mx = this; mxpos = x(i); end
  if this < mn, mn = this; mnpos = x(i); end
  
  if lookformax
    if this < mx-delta
      maxtab = [maxtab ; mxpos mx];
      mn = this; mnpos = x(i);
      lookformax = 0;
    end  
  else
    if this > mn+delta
      mintab = [mintab ; mnpos mn];
      mx = this; mxpos = x(i);
      lookformax = 1;
    end
  end
end
end