%% Diffussion analysis function
function [shoreline Difusion]=Terracem_terracediff(swat,station,profnum,shorenum,pto,thetaB,Dprof,thetadegrees,k,t1,ti,t2,varargin)
%==================================================================
%Julius J. 2015
%
% Use diffusion eq. of Hanks, 1989 to calculate geomorphic age of scarp
% and find the shoreline angle.
%==================================================================
%check inputs
if length(varargin) > 0
  
   tops1 = varargin{1};
   bots1 = varargin{2};
   scarp1 = varargin{3};
   inputset=1;
else
   inputset=0;
end   


resp=shorenum; %A) modify shorenum to input list
format long
x=swat(:,1); %distance along profile
mi=swat(:,2); %min values%
me=swat(:,3); %mean values
ma=swat(:,4); %max values
slp=swat(:,5); %slope values
% Check if flipped, high part must be at right
if ma(1)>ma(end)
   type_profile=2; %identifies if profile is flipped    
   x=max(x)-x+1;   
else   type_profile=1; %identifies if profile is in correct position 
    x=x;
end

f1=figure(1);
clf, hold on
ax1=subplot(3,1,1:2);
hold on
plot(x,ma,'k')
plot(x,me,'color',[0.3 0.3 0.3])
plot(x,mi,'color',[0.6 0.6 0.6])
hleg1 = legend('Max topography','Mean topography','Min topography','Location','SouthEast');
xlim([min(x) max(x)]);
ylabel('Elevation (m)'); 
xlabel('Distance along profile (m)');
tit=sprintf('Station %s, Profile %u, Shoreline %u',station,profnum,shorenum);
title(tit);
box on
ax2=subplot (3,1,3); 
hold on
plot(x,slp,'.k')
plot(x,moving(slp,50,'mean'),'r-');
leyendita = legend('slope values','Location','NorthEast');
xlim([min(x) max(x)]);
ylabel('Slope (deg)'); xlabel('Distance along profile (m)');
grid on 
box on
linkaxes([ax1 ax2], 'x');
%disp('Zoom in, select area, and press [Enter]')
%pause

if inputset==1;

bots=bots1;    
tops=tops1;
scarp=scarp1;

else

if exist(sprintf ('%s_Clicks.mat',station),'file')==2
load(sprintf ('%s_Clicks.mat',station));  %assumes that Clicks empty matrix was already saved  
else
Clicks=[]; %if clicks matrix not exist creates an empty  matrix for the next condition
end
%check structure existence
doesVarExist = true;
try
    Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff
catch
    doesVarExist = false;
end

if doesVarExist == 1
prompt1 = {'would you like to overwrite the data? Yes(0) / No (1)'};

dlg_title1 = 'rewrite or re-read the data?';
num_lines1 = 1;

def1 = {'1'}; %default values
answer1 = str2double(inputdlg(prompt1,dlg_title1,num_lines1,def1));

        if answer1(1,1)==0; 
        existence=0; %parameter for data existence situation, existence=0 no previous data recognized.
        %top point
        disp('...enter inflection point of far-field slope at top surface...');
        [tops(1,1),tops(1,2)]=ginputc(1,'Color','r');       
        %base points
        disp('...enter platform surface...');
        [bots(1,1),bots(1,2)]=ginputc(1,'Color','b'); 
        [bots(2,1),bots(2,2)]=ginputc(1,'Color','b');     
        else
        existence=1;  %parameter for data existence situation, existence=1 previous data recognized.  
        tops=Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.tops;
        bots=Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.bots;
       
        end
else
% MANUAL INPUTS
%top point
existence=0; 
disp('...enter inflection point of far-field slope at top surface...');
[tops(1,1),tops(1,2)]=ginputc(1,'Color','r');
%base points
disp('...enter bottom surface...');
[bots(1,1),bots(1,2)]=ginputc(1,'Color','b');
[bots(2,1),bots(2,2)]=ginputc(1,'Color','b');
end
axes(ax1)
 plot(tops(1,1),tops(1,2),'ob','markerfacecolor','k');
 plot(bots(1,1),bots(1,2),'ob','markerfacecolor','b');
 plot(bots(2,1),bots(2,2),'ob','markerfacecolor','b'); 
% new part for center point
center=bots(1,2)+((tops(1,2)-bots(1,2))/2);
plot([min(x) max(x)],[center center],'k--')

if existence==1
scarp=Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.scarp;
axes(ax1)
plot(scarp(1,1),scarp(1,2),'sb','markerfacecolor','g');
else
% center point
disp('...enter scarp center...');
[scarp(1,1),scarp(1,2)]=ginputc(1,'Color','g'); 

axes(ax1)
plot(scarp(1,1),scarp(1,2),'sb','markerfacecolor','g');
end
end
% fitting line at base with maximum topo
da(:,1)=x; %x axis now defined as da(:,1)
da(:,2)=ma; %fit respect to maximum topo values
da=sortrows(da,1); 
ny=length(da(:,1)); 
daplat1=da(da(:,1)<bots(1,1),:); %only at base
daplat=daplat1(daplat1(:,1)>bots(2,1),:);
clear daplat1
% linear regressions
% platform
xp=daplat(:,1); 
yp=daplat(:,2);
[pplat,splat]=polyfit(xp,yp,1);
% extrapolate platform
d=mean(diff(xp));
xx=min(xp):d:max(xp); xx=xx';
[p_plat,d_plat]=polyval(pplat,xx,splat);
%find x and y values
nr=numel(p_plat);
x1=xx(1,1); 
x2=xx(nr,1); 
y1=p_plat(1,1); 
y2=p_plat(nr,1); %rect eq.
%central point
x_scarp=scarp(1,1);
y_scarp=(((x_scarp-x1)*(y2-y1))/(x2-x1))+y1;
%top points(tops)
x_tops=tops(1,1);
y_tops=(((x_tops-x1)*(y2-y1))/(x2-x1))+y1;
%min value of x in profile
x_minx=min(da(:,1));
y_minx=(((x_minx-x1)*(y2-y1))/(x2-x1))+y1;
%table with basal line elements
bline(1,1)=x_scarp;
bline(1,2)=y_scarp;
bline(2,1)=x1;
bline(2,2)=y1;
bline(3,1)=x_minx;
bline(3,2)=y_minx;
%slope of far field lines %%%%% (b value comes from here)
ml=(y2-y1)/(x2-x1);
%line at top: with slope m1 passing by tops and x_max
%ec en la forma y_max-yt=m(x_max-xt) donde y_max es incognita
xt=tops(1,1); yt=tops(1,2);
x_max=max(da(:,1)); %topline at max x value
y_max=(ml*(x_max-xt))+yt;
x_scarp=scarp(1,1);
yt_scarp=(ml*(x_scarp-xt))+yt;
%table with topline elements
tline(1,1)=x_scarp;
tline(1,2)=yt_scarp;
tline(2,1)=xt;
tline(2,2)=yt;
tline(3,1)=x_max;
tline(3,2)=y_max;
%central vertical line that pass trough scarp 
%values of scarp in y at top and baseline 
v_line(1,1)=x_scarp;
v_line(1,2)=yt_scarp;
v_line(2,1)=x_scarp;
v_line(2,2)=y_scarp;
% precentered data plot %%%%%%%%%%%%%%%%%%
axes(ax1)
hold on
plot(bline(:,1),bline(:,2),'r--');
plot(tline(:,1),tline(:,2),'r--');
plot(v_line(:,1),v_line(:,2),'k','LineWidth',1);
text((v_line(1,1)),((v_line(1,2)+scarp(1,2))/2),'a','FontSize',9,'BackgroundColor','white')
text((v_line(1,1)),((v_line(2,2)+scarp(1,2))/2),'a','FontSize',9,'BackgroundColor','white')
%values of a height of scarp/2
a=(yt_scarp-y_scarp)/2;
%value of b far field slope
b=atand(ml);
text(bots(2,1),tops(1,2),['2a=',(num2str(2*a,'%0.1f')),' m'],'FontSize',12)
text(bots(2,1),((tops(1,2)+scarp(1,2))/2),['b=',(num2str(b,'%0.1f')),' ?'],'FontSize',12)
%save plot
rect = [5,6,13,18];
set(gcf,'PaperUnits','centimeters'); set(gcf, 'PaperType','A4'); set(gcf,'paperposition',rect);
fout = sprintf('%s_prof_%u_shore_%u_diffuse_picks.pdf',station,profnum,shorenum);
saveas(gcf,fout,'pdf');

% CENTERING THE ENTIRE PLOT DATA
%defining plot in cartesian plane
offx=x_scarp; %centerpoint x
offy=yt_scarp-a; %centerpoint y
%centering topografic data
nim=numel(da(:,1));
xn(:,1)=x(:,1)-offx;    
dan(:,1)=da(:,1)-offx; %for max topo analysis
dan(:,2)=da(:,2)-offy;
%centering far field slope data
num=numel(tline(:,1)); %numel for bline and tline
tlinen(:,1)=tline(:,1)-offx;
tlinen(:,2)=tline(:,2)-offy;
blinen(:,1)=bline(:,1)-offx;
blinen(:,2)=bline(:,2)-offy;
vlinen(:,1)=v_line(:,1)-offx;
vlinen(:,2)=v_line(:,2)-offy;
%centering bots and tops points of ginputs
topsn(1,1)=tops(1,1)-offx;
topsn(1,2)=tops(1,2)-offy;
botsn(1,1)=bots(2,1)-offx;
botsn(1,2)=bots(2,2)-offy;
scarpn(1,1)=scarp(1,1)-offx;
scarpn(1,2)=scarp(1,2)-offy;
%generating database "parameters"
parameters.offset.uperline=tlinen;
parameters.offset.lowerline=blinen;
parameters.offset.vline=vlinen;
parameters.profile.x=dan(:,1);
parameters.profile.maxtopo=dan;
parameters.profile.slope(:,1)=xn;
parameters.profile.slope(:,2)=slp;
parameters.inputs.a=a;
parameters.inputs.b=b;
parameters.profile.nim=nim; %nim=number of elements for dx calculation
%add ginputs point to paramaters to constrain the extension of rms analysis
parameters.inputs.topsn=topsn;
parameters.inputs.botsn=botsn;
parameters.inputs.scarp=scarpn;
%cut plot area using botsn and topsn
parameters.cut.maxtopo=parameters.profile.maxtopo; %vectorial form: first create a matrix equal that the one that i want to modify
parameters.cut.maxtopo(parameters.cut.maxtopo(:,1)>(parameters.inputs.topsn(1,1)+15),:)=[]; %vectorial form, excluve values bigger than x in topsn
parameters.cut.maxtopo(parameters.cut.maxtopo(:,1)<(parameters.inputs.botsn(1,1)-0),:)=[]; %vectorial form, excluve values bigger than x in botsn
nm=numel(parameters.cut.maxtopo(:,1));
parameters.cut.nim=nm;
clear x
%generate a sintetic distribution of the same size of the input topography
xmin=(parameters.cut.maxtopo(1,1));
dx=((parameters.cut.maxtopo(parameters.cut.nim,1)-parameters.cut.maxtopo(1,1))/(parameters.cut.nim-1));
xmax=(parameters.cut.maxtopo(parameters.cut.nim,1));
x=(xmin:dx:xmax);
a=parameters.inputs.a; %half oh the scarp elevation obtained from manual picking at the begining
bdegrees=parameters.inputs.b; %far field slope value, obtained from manual picking at the begining
%difussion loop
%thetadegrees
%k
t=t1:ti:t2; %range in time
ktrange=t.*k;
rms=zeros(numel(ktrange),1); %matris de zeros igual a ktrange
topopred=(parameters.cut.maxtopo(:,2))';

% RUN MODELS
j=1;
for kt=ktrange
model=finitescarp(thetadegrees,a,x,kt,bdegrees);
% RMS
rms(j)=sqrt(sum((topopred(:)-(-model(:))).^2)/numel(topopred));
j=j+1;
end
%plot RMS
rr=horzcat(ktrange',rms);
ir=sortrows(rr,2);
parameters.KT_RMS=ir;
best=parameters.KT_RMS(1,:);
% Shoreline angle extraction and error %%%%%%%
fface=[thetadegrees 5]; %angular rangle, mean=60 error =5
da2=parameters.cut.maxtopo;
daxclif=da2(da2(:,1)>=0,:); %change scarp por la inversa del shoreline angle para calcular variacion angular
xc=-daxclif(:,1); 
yc=(-tand(fface(1,1)).*abs(daxclif(:,1)-daxclif(1,1)))+daxclif(1,2);
yc1=daxclif(1,2)-((daxclif(:,1)-daxclif(1,1))./tand(90-fface(1,1)+fface(1,2)));
yc2=daxclif(1,2)-((daxclif(:,1)-daxclif(1,1))./tand(90-fface(1,1)-fface(1,2)));    
[pclif,sclif]=polyfit(xc,yc,1);
[pclif1,sclif1]=polyfit(xc,yc1,1);
[pclif2,sclif2]=polyfit(xc,yc2,1);
%platform
xg=daplat(:,1)-offx; %valores alcanzados por la interpolacion
yg=daplat(:,2)-offy;
[pplat,ssplat]=polyfit(xg,yg,1);
% extrapolate
dxm=mean(diff(xg));
%xxx=min(xg):d:(xxa+offx); 
xxx=min(xg):dxm:max(xc); 
xxx=xxx';
[pp_plat,dd_plat]=polyval(pplat,xxx,ssplat);
[p_clif,d_clif]=polyval(pclif,xxx,sclif);
% intersect
mc=pclif(1,1); 
ic=pclif(1,2); 
mp=pplat(1,1); 
ip=pplat(1,2);
sh=[(-ic+ip)/(mc-mp),(ip*mc-ic*mp)/(mc-mp)];
eshx=sh(1,1); 
eshz=sh(1,2);
[shzp,shzep]=polyval(pplat,eshx,ssplat);
% propagate 2s error by intersecting 2s regressions
% intersect max
mc=pclif1(1,1); 
ic=pclif1(1,2);
sh1=[(-ic+ip)/(mc-mp),(ip*mc-ic*mp)/(mc-mp)];
shx1=sh1(1,1); 
shz1=sh1(1,2);
[shzc1,shzec1]=polyval(pplat,shx1,splat);
% intersect min
mc=pclif2(1,1); 
ic=pclif2(1,2);
sh2=[(-ic+ip)/(mc-mp),(ip*mc-ic*mp)/(mc-mp)];
shx2=sh2(1,1); 
shz2=sh2(1,2);
[shzc2,shzec2]=polyval(pplat,shx2,splat);
% calculate error
shze1=shz1-shzec1;
shze2=shz2+shzec2;
shze=(shze2-shze1);
%transform xc and yc to original coordinates
xcyc(:,1)=xc; xcyc(:,2)=yc;
xcyc=xcyc(xcyc(:,2)>eshz,:);
xcyc1(:,1)=xc; xcyc1(:,2)=yc1;
xcyc1=xcyc1(xcyc1(:,2)>eshz,:);
xcyc2(:,1)=xc; xcyc2(:,2)=yc2;
xcyc2=xcyc2(xcyc2(:,2)>eshz,:);
%outputs
shoreline_angle_x=eshx+offx; %distance along profile
shoreline_angle_y=eshz+offy; %elevation
error_shoreline=2*shze; %error in elevation
%plot results

if inputset~=1;
pause(1.5)    
end

close(f1)
f2=figure(2);
clf
hinitial=finitescarp(thetadegrees,a, x, 0.0001,bdegrees);
bestfit=finitescarp(thetadegrees,a,x,best(1,1),bdegrees);
tit1=sprintf('Station %s, Profile %u, Shoreline %u',station,profnum,shorenum);
ax1=axes('position',[0.2 0.1 0.63 0.65]);
title(tit1)
hold on
box on
plot(x,-hinitial,'b--')
plot(parameters.cut.maxtopo(:,1),parameters.cut.maxtopo(:,2),'k','LineWidth',1.5)
plot(x,-bestfit,'r-')
hleg4 = legend('Initial condition','Topography','Best fit','location','SouthEast');
set(gca,'yaxislocation','left','tickdir','in')
set(gca,'xaxislocation','bottom','tickdir','in');
xlabel('Distance (m)')
ylabel('Elevation (m)')
xlim([(parameters.cut.maxtopo(1,1)) (parameters.cut.maxtopo(parameters.cut.nim,1))])
plot(eshx,eshz,'ok')
errorbar(eshx,eshz,shze,'dk','markerfacecolor','k')
% miniplot rms      
ax2=axes('position',[0.275 0.48 0.17 0.15]);
hold on
set(gca,'xaxislocation','bottom','tickdir','in');
plot(ktrange,rms,'b')
plot(best(1,1),best(1,2),'ok','markerfacecolor','k')
line([0 best(1,1)],[best(1,2) best(1,2)],'Linestyle','--','color','k');
line([best(1,1) best(1,1)],[0 best(1,2)],'Linestyle','--','color','k');
best_kt=best(:,1);
best_rms=best(:,2);
ak=round(best(1,1)+(best(1,1)/1));
ck=round(best(1,1)-(best(1,1)/1));
bk=round((best(1,2))+(best(1,2)/1));
lk=round(best(1,2)-(best(1,2)/1));
ylim([0 (best(1,2)*2)])
%xlim('auto')  
xlim([0 (best(1,1)*2)])
set(gca,'XTick',best(1,1))
set(gca,'YTick',best(1,2))
tit2=title('RMS misfit');
xlabel('Kt (m^2)')
ylabel('RMS (m)')

%sizing
v = axis;
xpos=v(1)+(v(2)-v(1))/2;
ypos=v(4)-((v(4)-v(3))/10);
set(tit2,'Position',[xpos ypos]);
%save figure as pdf
rect = [2,8,17,20];
set(gcf,'PaperUnits','centimeters'); set(gcf, 'PaperType','A4'); set(gcf,'paperposition',rect);
fout = sprintf('%s_prof_%u_shoreline_%u_diff.pdf',station,profnum,shorenum);
saveas(gcf,fout,'pdf');
clear p_clif p_plat pclif1 pclif2 pp_plat 


disp('fit saved')
if inputset~=1;
pause(1.5)    
end
close(f2)

%obtaining the UTM coordinate pairs of shoreline
p1(:,1)=shoreline_angle_x; %formatting iputs for point2utm
p1(:,2)=shoreline_angle_y; %formatting iputs for point2utm
point2utm=Terracem_point2utm(p1,pto,thetaB, Dprof, type_profile); %call to point2utm function
Rx=point2utm(:,1); %UTM E
Ry=point2utm(:,2); %UTM N
%Primary output
shoreline=[shorenum Rx Ry shoreline_angle_x shoreline_angle_y shze];
%secondary outputs
Difusion=horzcat(profnum,shorenum, Rx, Ry, shoreline_angle_y,shze,best_kt, best_rms, a, b);
%Clicks output
%check if table exist
if exist((sprintf ('%s_Clicks.mat',station)),'file')==2
%load table & combine structures
load(sprintf ('%s_Clicks.mat',station))
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.tops=tops;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.bots=bots;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.scarp=scarp;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.inputs.thetadegrees=thetadegrees;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.inputs.k=k;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.inputs.t1=t1;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.inputs.ti=ti;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.inputs.t2=t2;
save((sprintf ('%s_Clicks.mat',station)),'Clicks') 
else
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.tops=tops;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.bots=bots;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.scarp=scarp;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.inputs.thetadegrees=thetadegrees;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.inputs.k=k;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.inputs.t1=t1;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.inputs.ti=ti;
Clicks.(sprintf('prof_%u',profnum)).(sprintf('shore_%u',shorenum)).diff.inputs.t2=t2;
save((sprintf ('%s_Clicks.mat',station)),'Clicks') 
end
end


function H = finitescarp(thetadegrees,a, x, kt ,bdegrees)
% Solves the finitescarp initial conditions problem for
% thetadegrees is the initial scarp slope in degrees
% a = half scarp initial height
% x = distance along profile
% kt = morphologic age (L^2)
% bdegrees is far field slope in degrees
% from Hanks, T. C. and D. J. Andrews, Effect of far-field slope on morphologic dating 
% of scarplike landforms, Journal of Geophysical Research, 94, 565-573, 1989.
format long
theta=rads(thetadegrees);
b1=rads(bdegrees);
fkt=4.*kt;
sfkt=sqrt(fkt);
ang=theta-b1;
A=a./ang;
X=x+A;
Xm=x-A;
%Difussion core, coded by Arrowsmith, adapted by J. Jara-Munoz for use with swath profiles
    if theta>=rads(75)
   H=(a.*erf(x./(2.*sqrt(kt)))+b1.*x).*-1;%Note: -1 for flip profile
    else
   H=(ang.*sqrt((kt)/pi).*(exp(-((X.^2)./fkt))-exp(-((Xm.^2)./fkt)))+(ang/2).*(X.*erf(X./sfkt)-Xm .*erf(Xm./sfkt))+b1.*x).*-1;
    end
end    
%Small nested function, radian transformation
function outrads = rads(indeg)
format long
outrads = indeg.*pi/180;
end